/**************************************************************************************************
    아두이노 우노를 이용한 아두이노 미세먼지측정기 키트 센서(DSM501A) - DIY
  -.해당 소스는 출처 표기후 개인 또는 교육에 사용 가능한 오픈 소스입니다.
  -.아래 소스는 인터넷에서 수집한 정보로 기초로 수정한 소스 입니다.
  -.키트와 소스는 아두이노 DIY Tset의 목적으로 제공되며 정확성,신뢰성을 보장 하지는 않습니다.
  -.해당 소스를 사용하여 발생하는 모든 문제는 사용자 본인에게 책임이 있습니다.
  -.인터넷을 검색 하시면 더많은 다양한 예제를 보실수 있을 겁니다.
  -.3D 모델, 키트 주문제작, 소스코딩은 메일로 문의 하시기 바랍니다.
  -.문의: www.eduiot.kr / 010.5340.3088 / e-mail: iotmonstar@gmail.com
  -.Created/Modify by NA,S.B - 2017.v1
  ---------------------------------------------------------------------------------------------------*
*/
// *Wire Case-1(1~5 번호는 먼지센서 본체 콘넥터에 표기 되어있습니다)
// 코딩에 사용된 센서는 Wire Case-1 사용 하였습니다.
// Connect the Pin_3 of DSM501A to Arduino 5V
// Connect the Pin_5 of DSM501A to Arduino GND
// Connect the Pin_2 of DSM501A to Arduino D3


#include<string.h>
//2016(2x16) 키패드 쉴드 library
#include <LiquidCrystal.h>

//온습도 센서 library
#include <dht.h>         //  https://arduino-info.wikispaces.com/DHT11-Humidity-TempSensor
// Library down load  ==> https://arduino-info.wikispaces.com/file/view/DHT-lib.zip/545470280/DHT-lib.zip
/*-----( Declare objects )-----*/
dht DHT;
// 2016 LCD 패널에 통신을 위해 사용 된 핀번호.
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
int dly_button = 3000; // Button Push delay wait value.
byte buff[11];
unsigned long duration;  //지속시간
unsigned long starttime; //시작시간
unsigned long curtime;   //종료시간
unsigned long sampletime_ms = 30000; // 30초 = 먼지 측정 주기(간격)-조정하세요 ; (1초=1000으로 설정)
unsigned long lowpulseoccupancy = 0; //지속시간 누적
float ratio = 0;
float concentration = 0; // using spec sheet curve
float pcsPerCF = 0;  //입자당 CF를 0으로 초기화
float ugm3 = 0;  //세제곱미터(큐빅 미터)당 마이크로 그램(㎍/㎥)
int i = 0;
int dht_sw = 0;
int air_sw = 0;
/********************************************************************
  LCD panel and the buttons 
*********************************************************************/
// define some values used by the panel and buttons
int lcd_button_sw = 1; // LCD 버튼 스위치 값
int lcd_key     = 0;
int adc_key_in  = 0; // LCD 버튼의 누른(터치)값
// LCD 패널의 버튼에서 사용되는 값 정의
#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5

//온도 / 습도 아이콘 만들기 값 = 바이너리 값
//byte nan[8] = {B00000, B00000, B00000, B10010, B10010, B10010, B11110, B10000};
//byte pow[8] = {B11000, B00100, B11000, B00100, B11000, B00000, B00000, B00000};

// Custom character - i1 / 동일한 기능을 하는 값을 두가지로 작성 하였습니다.
byte nan[8] = //icon for termometer 온도계 아이콘
{ B00100,  B01010,  B01010,  B01110,  B01110,  B11111,  B11111,  B01110};
uint8_t nan8[8] = { 0x4,     0xa,     0xa,     0xe,     0xe,     0x1f,    0x1f,    0xe   };

// Custom character - i2 / 동일한 기능을 하는 값을 두가지로 작성 하였습니다.
byte pic[8] = //icon for water droplet 습도계 아이콘
{ B00100,  B00100,  B01010,  B01010,  B10001,  B10001,  B10001,  B01110};
uint8_t pic8[8] = { 0x4,     0x4,     0xa,     0xa,     0x11,     0x11,   0x11,    0xe   };

int pin = 3;//DSM501A input pin#
/*-----( Declare Constants, Pin Numbers )-----*/
#define DHT11_PIN 2 //dht11 signal pin connected to pin.
/*********************************************************************************************
   setup - 이 처리 루틴은 아두이노가 시작후 처음 한번만 실행후 - 다음 Loop를 계속 처리를 합니다
 *********************************************************************************************/
void setup()
{
  pinMode(pin, INPUT);
  Serial.begin(115200); //시리얼 통신 데이타를 확인할수 있습니다.
  lcd.begin(16, 2);       //Creating an Object and naming it as "lcd".
  lcd.clear();
  Serial.println("Dust senser start,\twait,...\n");
  // i1).온도 / 습도 아이콘  생성, 바이너리(Binary) 값 = 아래의 i2).헥사(Hex) 값과 동일한 기능을 합니다.
  // Custom character - 1
  /*
    lcd.createChar(8, nan); //온도 아이콘 만들기
    lcd.createChar(9, pic); //습도 아이콘 만들기
    lcd.setCursor(4, 1);           // LCD 위치 0,0 : x=칼럼,y=라인(열)
    lcd.print((char)8);     // 온도 아이콘 LCD에 출력
    lcd.setCursor(8, 1);           // LCD 위치 0,0 : x=칼럼,y=라인(열)
    lcd.print((char)9);     // 습도 아이콘 LCD에 출력
  */
  /*
    //i2). 온도 / 습도 아이콘 내부 버퍼에 생성, 헥사(Hex) 값 = 위의 i1).바이너리(Binary) 값과 동일한 기능을 합니다.
    // Custom character - 2
    lcd.createChar(8, nan8); //온도 아이콘 만들기
    lcd.createChar(9, pic8); //습도 아이콘 만들기

    lcd.setCursor(4, 1);           // LCD 위치 0,0 : x=칼럼,y=라인(열)
    lcd.write(8);            // 온도 아이콘 LCD에 출력
    lcd.setCursor(8, 1);           // LCD 위치 0,0 : x=칼럼,y=라인(열)
    lcd.write(9);            // 습도 아이콘 LCD에 출력
  */
  lcd.createChar(8, nan8); //온도 아이콘 만들기
  lcd.createChar(9, pic8); //습도 아이콘 만들기
  pinMode(pin, INPUT);  // 생략가능
  starttime = millis(); //시작 값을 아두이노로부터 읽어 들인다.- 초 단위로 사용

  lcd.begin(16, 2);              // 로고 출력후 LCD 초기화.
  lcd.clear();
}
/****************************************************************************************
   Loop - 이 처리 루틴은 아두이노가 Setup 실행후 - 종료 처리를 하지 않는 이상 계속 처리(루핑)를 합니다
 ****************************************************************************************/
void loop()
{
  //--------- 먼지 센서 값을 검출 계산 --------------------------------------------*/
  duration = pulseIn(pin, LOW);
  lowpulseoccupancy = lowpulseoccupancy + duration;
  curtime = millis(); //아두이노 내부, 현재 시간;
  if ((curtime - starttime) >= sampletime_ms) //대기시간 > = 30s
  {
    ratio = lowpulseoccupancy / (sampletime_ms * 10.0);  // // 비율 정수값 0=>100
    concentration = 1.1 * pow(ratio, 3) - 3.8 * pow(ratio, 2) + 520 * ratio + 0.62; // using spec sheet curve
    pcsPerCF = concentration * 100;  // 입자 농도에 100을 곱하면 입자당 CF값
    ugm3 = pcsPerCF / 13000;  //입자당 CF를 13000으로 나누면 세제곱미터당 마이크로그램 미세먼지 값

    // Serial.print("duration:");
    // Serial.print(duration);
    // Serial.print(" / ");
    // Serial.print("ugm3:");
    // Serial.print(ugm3);
    // Serial.print(" / ");
    // Serial.print("lowpulseoccupancy:");
    // Serial.print(lowpulseoccupancy);
    // Serial.print(" / ");
    // Serial.print("ratio:");
    // Serial.print(ratio);
    // Serial.print(" / ");
    // Serial.print("concentration:");
    // Serial.print(concentration);
    // Serial.print(" / ");
    // Serial.print(" pcs/0.01cf:");
    // Serial.println(pcsPerCF);
    // Serial.println("\n");
    // Serial.print("f-concentration:");
    // Serial.println(float(concentration/500));
    lowpulseoccupancy = 0;
    starttime = millis();
  }

  if ( concentration == 0)  {    //만약 결과값이 0 이면 아래를 LCD에 출력한다.
    lcd.setCursor(0, 0);
    lcd.print("Analyzing air:"); // 메세지 출력.
    lcd.print(air_sw++); lcd.print(" ");
    lcd.blink();
    delay (150);
  } else
  {
    air_sw = 0;
    lcd.setCursor(0, 0);
    lcd.print("Dust ");
    lcd.print(ugm3);
//    lcd.print(float(concentration/500));
    lcd.print(" ug/m3");         lcd.print(" ");
    lcd.noBlink();
  }
  
  //  1602 keypad Shield BUTTON 눌렀는지 확인
  lcd_button_sw = 0;            // 버튼 스위치 초기화
  lcd_key = read_LCD_buttons();  // read the buttons
  switch (lcd_key)               // 어떤 버튼을 눌렀는지 값을 확인후 해당 버튼 값을 LCD에 출력, RESET 버튼은 출력 안함=바로 리부팅(재시작).
  {
    case btnRIGHT:
      {
        init_lcd();
        lcd.print("Push on button");   // LCD에 출력.
        lcd.setCursor(4, 1);           // LCD 위치 조정 : x=칼럼,y=라인(열)
        lcd.print("RIGHT ");
        delay(dly_button);
        lcd_button_sw = 1;
        lcd.clear();
        break;
      }
    case btnLEFT:
      {
        init_lcd();
        lcd.print("Push on button");   // LCD에 출력.
        lcd.setCursor(4, 1);           // LCD 위치 조정 : x=칼럼,y=라인(열)
        lcd.print("LEFT   ");
        lcd_button_sw = 2;
        delay(dly_button);
        lcd.clear();
        break;
      }
    case btnUP:
      {
        init_lcd();
        lcd.print("Push on button");   // LCD에 출력.
        lcd.setCursor(6, 1);           // LCD 위치 조정 : x=칼럼,y=라인(열)
        lcd.print("UP    ");
        lcd_button_sw = 3;
        delay(dly_button);
        lcd.clear();
        break;
      }
    case btnDOWN:
      {
        init_lcd();
        lcd.print("Push on button");   // LCD에 출력.
        lcd.setCursor(4, 1);           // LCD 위치 조정 : x=칼럼,y=라인(열)
        lcd.print("DOWN  ");
        lcd_button_sw = 4;
        delay(dly_button);
        lcd.clear();
        break;
      }
    case btnSELECT:
      {
        init_lcd();
        lcd.print("Push on button");   // LCD에 출력.
        lcd.setCursor(4, 1);           // LCD 위치 조정 : x=칼럼,y=라인(열)
        lcd.print("SELECT");
        lcd_button_sw = 5;
        delay(dly_button);
        lcd.clear();
        break;
      }
    case btnNONE:
      {
        //        lcd.print("NONE  ");
        //        delay(dly_button);
        lcd_button_sw = 0;
        break;
      }
  } // - end case

  if (lcd_key = btnNONE)   { // 버튼이 눌러지지 않은 경우만 온,습도 처리.
    dht_sw = 0;
    // READ DATA
    int chk = DHT.read11(DHT11_PIN); // 온도,습도 센서의 값을 읽음.
    switch (chk)
    {
      case DHTLIB_OK:
        //        Serial.print("OK,\t");
        break;
      case DHTLIB_ERROR_CHECKSUM:
        Serial.print("Checksum error,\t");
        dht_sw = 1;
        break;
      case DHTLIB_ERROR_TIMEOUT:
        Serial.print("Time out error,\t");
        dht_sw = 1;
        break;
      case DHTLIB_ERROR_CONNECT:
        Serial.print("Connect error,\t");
        dht_sw = 1;
        break;
      case DHTLIB_ERROR_ACK_L:
        Serial.print("Ack Low error,\t");
        dht_sw = 1;
        break;
      case DHTLIB_ERROR_ACK_H:
        Serial.print("Ack High error,\t");
        dht_sw = 1;
        break;
      default:
        Serial.print("Unknown error,\t");
        dht_sw = 1;
        break;
    }
        dsp_Temp_Hum(dht_sw); //displaying Temperature & Humidity -- 온도와 습도를 LCD에 출력.
  }


  delay(1000); // 1초 대기
}

///////////////// - END OF SOURCE (main-loop) - //////////////////////////


// ----------------- Begin Function -----------------------------------//

/***********************************************************************
    // 온도  습도 센서의 값을 화면의 2번째 라인에 출력 합니다.
 ***********************************************************************/
void dsp_Temp_Hum(int dht_sw)
{ //Serial.print("dsp_Temp_Hum:"); Serial.println(dht_sw);
  int chk = DHT.read11(DHT11_PIN); // 온도,습도 센서의 값을 읽음.
  lcd.setCursor(0, 1);
  lcd.write(8);                         //온도 아이콘 앞쪽
  if (dht_sw == 0) {
    lcd.setCursor(2, 1);
    lcd.print("");
    lcd.print((float)DHT.temperature, 0); //온도출력
    //     lcd.setCursor(4, 1);
    lcd.print((char)223); //온도표시 아이콘 출력(뒤쪽)
    lcd.print("C");
  } else { // 센서 오류
    lcd.setCursor(2, 1);
    lcd.print(" ");
    lcd.print("^-^"); // 아이콘 출력(뒤쪽)
    lcd.print(" ");      lcd.print(" ");      lcd.print(" ");
  }


  lcd.setCursor(9, 1);
  lcd.write(9);                      //습도 아이콘 앞쪽
  if (dht_sw == 0) {
    lcd.setCursor(11, 1);
    lcd.print((float)DHT.humidity, 0); //습도 출력
    //  lcd.setCursor(13, 1);
    lcd.print("%");
    lcd.print(" ");
  } else { // 센서 오류
    lcd.setCursor(11, 1);
    lcd.print(" ");
    lcd.print("^-^"); // 아이콘 출력(뒤쪽)
  }

  // DHT11 센서는 처리후, 최소 1초의 시간은 기다려야 합니다.
  // 현재의 소스는 먼지 센서 부분에서 대기 시간 설정 있음.
}
/***********************************************************************
    // read the buttons 버튼을 눌렀는지 체크 합니다.
 ***********************************************************************/
int read_LCD_buttons()
{
  //  Serial.println("Push Button check#:");

  adc_key_in = analogRead(0);      // read the value from the sensor

  if (adc_key_in > 1000) return btnNONE;
  if (adc_key_in < 50)   return btnRIGHT;
  if (adc_key_in < 195)  return btnUP;
  if (adc_key_in < 380)  return btnDOWN;
  if (adc_key_in < 555)  return btnLEFT;
  if (adc_key_in < 790)  return btnSELECT;
  return btnNONE;  // when all others fail, return this...
}

/***********************************************************************
  // LCD 화면 초기화
  *********************************************************************/
void init_lcd() {
  lcd.begin(16, 2);              // LCD 초기화.
  lcd.setCursor(0, 0);           // LCD 위치 조정 : x=칼럼,y=라인(열)
  lcd.clear();
}

